import pandas as pd
import argparse
import sys
import matplotlib.pyplot as plt
import numpy as np
import statistics

sys.stdout = open("output.txt", "w")


parser = argparse.ArgumentParser(description="Enter the dataset : ")
parser.add_argument(
    "-dataset",
    "--dataset",
    type=str,
    required=True,
    help="DATASET1 or DATASET2",
)

args = parser.parse_args()
# making dataframe
df = pd.read_csv(args.dataset)


def print_dict(dict):
    for k, v in dict.items():
        print(k, "\n", v)


def bar_plot_tpo_2_ips():
    # initialize a dict with keys the dest ip addr and values 0
    frequency_dict_ips = {}
    for ip_dest in df["Destination"]:
        if ip_dest == str(ip_dest):  # type('nan') == float
            frequency_dict_ips[ip_dest] = 0

    # find the frequncies of the dest ip addr
    for ip_dest in df["Destination"]:
        if ip_dest == str(ip_dest):  # type('nan') == float
            frequency_dict_ips[ip_dest] += 1

    # sort the dict by value
    sorted_frequency_dict_ips = sorted(
        frequency_dict_ips.items(), key=lambda x: x[1], reverse=True
    )
    sorted_frequency_dict_ips = dict(sorted_frequency_dict_ips)
    # get the first 2 pairs from the dict
    sorted_frequency_dict_ips = dict(list(sorted_frequency_dict_ips.items())[:2])

    fig = plt.figure(figsize=(10, 5))
    plt.bar(
        list(sorted_frequency_dict_ips.keys()),
        list(sorted_frequency_dict_ips.values()),
        width=0.4,
    )

    plt.ylabel("Number of packets", fontweight="bold")
    plt.title("Top-2 frequent Destination IPs")
    plt.grid(linestyle="dotted")
    plt.show()


# pie chart
def pie_chart_with_protocols_except_UDP():
    # initialize a dict with keys the different ptotocols and values 0
    frequency_dict_protocols = {}
    for protocol in df["Protocol"]:
        if protocol != "UDP":
            frequency_dict_protocols[protocol] = 0

    # find the frequncies of the different protocols
    for protocol in df["Protocol"]:
        if protocol != "UDP":
            frequency_dict_protocols[protocol] += 1

    y = np.array(list(frequency_dict_protocols.values()))
    mylabels = list(list(frequency_dict_protocols.keys()))

    plt.pie(y, labels=mylabels)
    plt.title("The percentage of different protocols except UDP")
    plt.show()


# bar plot that represents the number
# of packets that the IP 63.111.11.187 received per second


def bar_plot_seconds():
    new_df = df[df.Destination == "63.111.11.187"]

    num_of_packets_per_second = {}
    for time in new_df["Time"]:
        num_of_packets_per_second[(str(time)[: str(time).index(".")])] = 0

    for time in new_df["Time"]:
        num_of_packets_per_second[(str(time)[: str(time).index(".")])] += 1

    plt.bar(
        list(num_of_packets_per_second.keys()), list(num_of_packets_per_second.values())
    )
    plt.grid(linestyle="dotted")
    plt.ylabel("Number of packets", fontweight="bold")
    plt.title("The number of packets that the IP 63.111.11.187 received per time")
    plt.xlabel("Seconds", fontweight="bold")
    plt.show()


def histogram():
    packet_lengths = [int(length) for length in df["Length"]]
    print(statistics.median(packet_lengths))

    plt.hist(packet_lengths)
    plt.grid(linestyle="dotted")
    plt.xlabel("Packet length", fontweight="bold")
    plt.ylabel("Number of packets", fontweight="bold")
    plt.show()


bar_plot_tpo_2_ips()
pie_chart_with_protocols_except_UDP()
bar_plot_seconds()
histogram()
